"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CliOutput = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _workspace() {
  const data = require("@teambit/workspace");

  _workspace = function () {
    return data;
  };

  return data;
}

function _ui() {
  const data = require("@teambit/ui");

  _ui = function () {
    return data;
  };

  return data;
}

function _webpack() {
  const data = require("@teambit/webpack");

  _webpack = function () {
    return data;
  };

  return data;
}

function _bundler() {
  const data = require("@teambit/bundler");

  _bundler = function () {
    return data;
  };

  return data;
}

function _compiler() {
  const data = require("@teambit/compiler");

  _compiler = function () {
    return data;
  };

  return data;
}

function _react() {
  const data = _interopRequireDefault(require("react"));

  _react = function () {
    return data;
  };

  return data;
}

function _ink() {
  const data = require("ink");

  _ink = function () {
    return data;
  };

  return data;
}

function _open() {
  const data = _interopRequireDefault(require("open"));

  _open = function () {
    return data;
  };

  return data;
}

function _outputTemplates() {
  const data = require("./output-templates");

  _outputTemplates = function () {
    return data;
  };

  return data;
}

/* eslint-disable @typescript-eslint/no-floating-promises */
// Import the IDs & Events
class CliOutput extends _react().default.Component {
  constructor(props) {
    var _props$uiServer;

    super(props);
    (0, _defineProperty2().default)(this, "isBrowserOpen", false);
    (0, _defineProperty2().default)(this, "eventsListener", event => {
      switch (event.type) {
        case _bundler().ComponentsServerStartedEvent.TYPE:
          this.updateOrAddComponentServer(event.data.context.id, 'Starting...', event.data);
          this.safeOpenBrowser();
          break;

        case _webpack().WebpackCompilationDoneEvent.TYPE:
          this.onWebpackCompilationDone(event);
          break;

        case _ui().UiServerStartedEvent.TYPE:
          this.onUiServerStarted(event);
          break;

        case _workspace().OnComponentChangeEvent.TYPE:
          this.onComponentChange(event);
          break;

        case _workspace().OnComponentAddEvent.TYPE:
          this.onComponentAdd(event);
          break;

        case _workspace().OnComponentRemovedEvent.TYPE:
          this.onComponentRemoved(event);
          break;

        case _compiler().CompilerErrorEvent.TYPE:
          this.setState({
            latestError: event.data.error,
            compiling: false
          });
          break;

        default:
      }
    });
    (0, _defineProperty2().default)(this, "onWebpackCompilationDone", event => {
      const successfullyCompiledComponents = event.data.stats.compilation.errors.length ? [] : [event.data.stats.hash];
      this.setState({
        webpackErrors: [...event.data.stats.compilation.errors],
        webpackWarnings: [...event.data.stats.compilation.warnings],
        compiledComponents: [...this.state.compiledComponents, ...successfullyCompiledComponents],
        compiling: false
      });
      this.updateOrAddComponentServer(event.data.devServerID, 'Running');
      this.safeOpenBrowser();
    });
    (0, _defineProperty2().default)(this, "onComponentRemoved", event => {
      this.onComponentChange(event);
    });
    (0, _defineProperty2().default)(this, "onComponentAdd", event => {
      this.onComponentChange(event);
    });
    this.state = {
      compiledComponents: [],
      commandFlags: props.commandFlags,
      mainUIServer: props.uiServer,
      componentServers: [],
      componentChanges: [],
      latestError: null,
      webpackErrors: [],
      webpackWarnings: [],
      totalComponents: null,
      isScope: !!((_props$uiServer = props.uiServer) === null || _props$uiServer === void 0 ? void 0 : _props$uiServer.uiRoot.scope),
      compiling: false
    };
    this.registerToEvents(props.pubsub);
  }

  registerToEvents(pubsub) {
    pubsub.sub(_ui().UIAspect.id, this.eventsListener);
    pubsub.sub(_webpack().WebpackAspect.id, this.eventsListener);
    pubsub.sub(_bundler().BundlerAspect.id, this.eventsListener);
    pubsub.sub(_workspace().WorkspaceAspect.id, this.eventsListener);
    pubsub.sub(_compiler().CompilerAspect.id, this.eventsListener);
  }

  async onUiServerStarted(event) {
    const devServers = await event.data.uiRoot.devServers;

    if (event.data.uiRoot.scope) {
      this.setState({
        mainUIServer: event.data,
        isScope: true
      });
    } else {
      const totalComponents = await event.data.uiRoot.workspace.list();
      devServers.forEach(server => {
        this.updateOrAddComponentServer(server.context.id, 'Starting...', server);
      });
      this.setState({
        mainUIServer: event.data,
        totalComponents
      });
    }

    if (!devServers.length) {
      this.unsafeOpenBrowser(this.state.mainUIServer);
    } else {
      this.safeOpenBrowser();
    }
  }

  onComponentChange(event) {
    this.clearConsole();
    this.setState({
      componentChanges: [...this.state.componentChanges, event],
      latestError: null,
      webpackErrors: [],
      webpackWarnings: [],
      compiling: true
    });
  }

  // Helpers
  clearConsole() {
    process.stdout.write(process.platform === 'win32' ? '\x1B[2J\x1B[0f' : '\x1B[2J\x1B[3J\x1B[H');
  }

  async safeOpenBrowser() {
    const {
      suppressBrowserLaunch
    } = this.state.commandFlags;
    const {
      mainUIServer
    } = this.state;

    if (mainUIServer && !suppressBrowserLaunch && !!this.state.compiledComponents.length) {
      // TODO(uri): Bug two events for each server
      if (this.state.compiledComponents.length / 2 >= this.state.componentServers.length) {
        this.unsafeOpenBrowser(mainUIServer);
      }
    }
  }

  unsafeOpenBrowser(mainUIServer) {
    const {
      suppressBrowserLaunch
    } = this.state.commandFlags;

    if (!this.isBrowserOpen && !suppressBrowserLaunch) {
      this.isBrowserOpen = true;
      setTimeout(() => (0, _open().default)(`http://${mainUIServer.targetHost}:${mainUIServer.targetPort}/`), 500);
    }
  }

  updateOrAddComponentServer(id, status, server) {
    if (server) {
      this.addOrUpdateComponentServer(id, status, server);
    } else {
      this.updateComponentServerStatus(id, status);
    }
  }

  updateComponentServerStatus(id, status) {
    var _this$state$component;

    const server = (_this$state$component = this.state.componentServers.find(cs => cs.id === id)) === null || _this$state$component === void 0 ? void 0 : _this$state$component.server;

    if (server) {
      this.addOrUpdateComponentServer(id, status, server);
    }
  }

  addOrUpdateComponentServer(id, status, server) {
    this.setState({
      componentServers: [...this.state.componentServers.filter(cs => cs.id !== id), {
        server,
        id,
        status
      }]
    });
  }

  render() {
    const {
      componentServers,
      mainUIServer,
      latestError,
      webpackErrors,
      webpackWarnings,
      compiledComponents,
      isScope,
      compiling
    } = this.state;
    const {
      verbose
    } = this.state.commandFlags; // run in scope

    if (isScope) {
      (0, _ink().render)( /*#__PURE__*/_react().default.createElement(_outputTemplates().UIServersAreReadyInScope, {
        mainUIServer: mainUIServer
      }));
      return null;
    }

    if (latestError) {
      return /*#__PURE__*/_react().default.createElement(_outputTemplates().TSErrors, {
        latestError: latestError,
        verbose: !!verbose
      });
    }

    if (webpackErrors.length) {
      return /*#__PURE__*/_react().default.createElement(_outputTemplates().WebpackErrors, {
        errs: webpackErrors,
        verbose: !!verbose
      });
    }

    if (webpackWarnings.length) {
      return /*#__PURE__*/_react().default.createElement(_outputTemplates().WebpackWarnings, {
        warnings: webpackWarnings,
        verbose: !!verbose
      });
    }

    if (compiling) {
      return /*#__PURE__*/_react().default.createElement(_ink().Text, null, "Compiling...");
    }

    return /*#__PURE__*/_react().default.createElement(_react().default.Fragment, null, mainUIServer ? null : /*#__PURE__*/_react().default.createElement(_outputTemplates().Starting, {
      componentServers: componentServers
    }), /*#__PURE__*/_react().default.createElement(_ink().Newline, null), /*#__PURE__*/_react().default.createElement(_outputTemplates().ComponentPreviewServerStarted, {
      items: componentServers
    }), /*#__PURE__*/_react().default.createElement(_ink().Newline, null), /*#__PURE__*/_react().default.createElement(_outputTemplates().CompilingOrUIServersAreReady, {
      totalComponentsSum: this.state.componentServers.length,
      compiledComponentsSum: compiledComponents.length,
      mainUIServer: mainUIServer
    }));
  }

}

exports.CliOutput = CliOutput;

//# sourceMappingURL=start.cmd/cli-output.js.map